@@ -1,6 +1,6 @@
 #!/usr/bin/env perl
 #
-# Copyright 2010-2019, Julian Catchen <jcatchen@illinois.edu>
+# Copyright 2010-2020, Julian Catchen <jcatchen@illinois.edu>
 #
 # This file is part of Stacks.
 #
@@ -34,10 +34,12 @@ my $popmap_path  = "";
 my $sample_path  = "";
 my $db           = "";
 my $min_cov      = 0;
-my $min_rcov     = 0;
 my $sample_id    = 1;
 my $gzip         = false;
 my $paired       = false;
+my $resume       = false;
+my $force_reexe  = false;
+my $genetic_map  = false;
 my $time         = "";
 
 my @parents;
@@ -61,7 +63,7 @@ my ($log, $log_fh, $sample);
 
 my (@sample_list, %pop_ids, %pops, %grp_ids, %grps, %sample_ids);
 
-parse_population_map(\@sample_list, \%pop_ids, \%pops, \%grp_ids, \%grps);
+parse_population_map(\@sample_list, \%pop_ids, \%pops, \%grp_ids, \%grps, \$genetic_map);
 
 initialize_samples(\@parents, \@progeny, \@samples, \@sample_list, \%pop_ids, \%grp_ids);
 
@@ -69,17 +71,32 @@ initialize_samples(\@parents, \@progeny, \@samples, \@sample_list, \%pop_ids, \%
 # Open the log file
 #
 $log = "$out_path/denovo_map.log";
-open($log_fh, ">$log") or die("Unable to open log file '$log'; $!\n");
 
-print $log_fh
+#
+# If we are resuming, check if the log exists and append to it.
+#
+if ($resume == true) {
+    open($log_fh, ">>$log") or die("Unable to open log file '$log'; $!\n");
+
+    print $log_fh
+        "\n",
+        "denovo_map.pl version ", stacks_version, " resuming at ", strftime("%Y-%m-%d %H:%M:%S", (localtime(time))), "\n",
+        $cmd_str, "\n";
+
+} else {
+    open($log_fh, ">$log") or die("Unable to open log file '$log'; $!\n");
+
+    print $log_fh
         "denovo_map.pl version ", stacks_version, " started at ", strftime("%Y-%m-%d %H:%M:%S", (localtime(time))), "\n",
         $cmd_str, "\n";
+}
 
 execute_stacks($log_fh, $sample_id, \@parents, \@progeny, \@samples, \%sample_ids);
 
 print $log_fh "\ndenovo_map.pl completed at ", strftime("%Y-%m-%d %H:%M:%S", (localtime(time))), "\n";
 close($log_fh);
 
+
 sub check_return_value {
     # $? is a 16 bit int. Exit code is given by `$? & 255` if the process was
     # terminated by a signal, and by `$? >> 8` if it exited normally.
@@ -109,11 +126,10 @@ sub check_return_value {
 sub execute_stacks {
     my ($log_fh, $sample_id, $parents, $progeny, $samples, $sample_ids) = @_;
 
-    my (@results, @depths_of_cov);
+    my (@results, @depths_of_cov, @prev_completed_ustacks);
     my ($pop_cnt, $sample, $num_files, $i, $cmd, $pipe_fh);
 
     my $minc  = $min_cov  > 0 ? " -m $min_cov"  : "";
-    my $minrc = $min_rcov > 0 ? " -m $min_rcov" : $minc;
 
     $i         = 1;
     $num_files = scalar(@{$parents}) + scalar(@{$progeny}) + scalar(@{$samples});
@@ -123,7 +139,7 @@ sub execute_stacks {
     #
     print STDERR "Indentifying unique stacks...\n";
     print $log_fh "\nustacks\n==========\n";
-    foreach $sample (@parents, @progeny, @samples) {
+    foreach $sample (@{$parents}, @{$progeny}, @{$samples}) {
 
         print $log_fh "\nSample $i of $num_files '$sample->{'file'}'\n----------\n";
 
@@ -131,14 +147,20 @@ sub execute_stacks {
             $sample_id = $sample_ids->{$sample->{'file'}};
         }
 
-        $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $sample->{'path'} -o $out_path -i $sample_id";
-        if ($sample->{'type'} eq "sample") {
-            $cmd .= $minc;
-        } elsif ($sample->{'type'} eq "parent") {
-            $cmd .= $minc;
-        } elsif ($sample->{'type'} eq "progeny") {
-            $cmd = $minrc;
+        #
+        # If resuming the pipeline, test if this sample already has been processed.
+        #
+        if ($resume == true and ustacks_sample_exists($sample, $out_path) == true) {
+            push(@prev_completed_ustacks, $sample);
+            print STDERR  "  Sample '$sample->{'file'}' already exists, will use existing data.\n";
+            print $log_fh "Sample '$sample->{'file'}' already exists, will use existing data.\n";
+            $i++;
+            $sample_id++;
+            next;
         }
+
+        $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $sample->{'path'} -o $out_path -i $sample_id" . $minc;
+
         if ($sample->{'path'} !~ /$sample->{'file'}.$sample->{'suffix'}$/) {
             # Guessing the sample name from the input path won't work.
             $cmd .= " --name " . $sample->{'file'};
@@ -180,18 +202,56 @@ sub execute_stacks {
     print STDERR "Generating catalog...\n";
     print $log_fh "\ncstacks\n==========\n";
 
-    $cmd = $exe_path . "cstacks -P $out_path " . join(" ", @_cstacks);
-    print STDERR  "  $cmd\n\n";
-    print $log_fh "$cmd\n\n";
-
-    if ($dry_run == false) {
-        open($pipe_fh, "$time $cmd 2>&1 |");
-        while (<$pipe_fh>) {
-            print $log_fh $_;
-            if ($_ =~ /failed/i) { print STDERR "Catalog construction failed.\n"; exit(1); }
+    
+    #
+    # If a genetic map was specified (based on the population map), only place
+    # samples that are 'parent' into the catalog.
+    #
+    if ($genetic_map == true) {
+        my @params = ();
+        while (@_cstacks) {
+            my $param = pop(@_cstacks);
+            next if (substr($param, 0, 2) eq "-M");
+            push(@params, $param);
+        }
+        push(@_cstacks, @params);
+
+        foreach $sample (@{$parents}) {
+            push(@_cstacks, "-s " . $out_path . "/" . $sample->{'file'});
+        }
+        push(@_cstacks, "-o $out_path");
+
+    } else {
+        push(@_cstacks, "-P $out_path");
+    }
+
+    #
+    # If the pipeline is in --resume mode, check if all ustacks samples existed previously and
+    # if the catalog already exists.
+    #
+    if ($resume == true and catalog_exists(\@prev_completed_ustacks, $parents, $progeny, $samples, $out_path) == true) {
+        print STDERR  "  Catalog already exists, will use existing data.\n\n";
+        print $log_fh "Catalog already exists, will use existing data.\n\n";
+
+    } else {
+        #
+        # If --resume is enabled and we ended up re-executing cstacks, force everything downstream to re-execute.
+        #
+        $force_reexe = true if ($resume == true);
+        
+        $cmd = $exe_path . "cstacks " . join(" ", @_cstacks);
+        print STDERR  "  $cmd\n\n";
+        print $log_fh "$cmd\n\n";
+
+        if ($dry_run == false) {
+            open($pipe_fh, "$time $cmd 2>&1 |");
+            while (<$pipe_fh>) {
+                print $log_fh $_;
+                if ($_ =~ /failed/i) { print STDERR "Catalog construction failed.\n"; exit(1); }
+            }
+            close($pipe_fh);
+            check_return_value($?, $log_fh);
         }
-        close($pipe_fh);
-        check_return_value($?, $log_fh);
     }
 
     #
@@ -200,17 +260,28 @@ sub execute_stacks {
     print STDERR "Matching samples to the catalog...\n";
     print $log_fh "\nsstacks\n==========\n";
 
-    $cmd      = $exe_path . "sstacks -P $out_path " . join(" ", @_sstacks);
-    print STDERR  "  $cmd\n\n";
-    print $log_fh "$cmd\n\n";
+    if ($resume == true and sstacks_samples_exist($parents, $progeny, $samples, $out_path) == true) {
+        print STDERR  "  sstacks matches already exist, will use existing data.\n\n";
+        print $log_fh "sstacks matches already exist, will use existing data.\n\n";
 
-    if ($dry_run == false) {
-        open($pipe_fh, "$time $cmd 2>&1 |");
-        while (<$pipe_fh>) {
-            print $log_fh $_;
+    } else {
+        #
+        # If --resume is enabled and we ended up re-executing sstacks, force everything downstream to re-execute.
+        #
+        $force_reexe = true if ($resume == true);
+        
+        $cmd      = $exe_path . "sstacks -P $out_path " . join(" ", @_sstacks);
+        print STDERR  "  $cmd\n\n";
+        print $log_fh "$cmd\n\n";
+
+        if ($dry_run == false) {
+            open($pipe_fh, "$time $cmd 2>&1 |");
+            while (<$pipe_fh>) {
+                print $log_fh $_;
+            }
+            close($pipe_fh);
+            check_return_value($?, $log_fh);
         }
-        close($pipe_fh);
-        check_return_value($?, $log_fh);
     }
 
     #
@@ -219,24 +290,35 @@ sub execute_stacks {
     print STDERR "Sorting reads by RAD locus...\n";
     print $log_fh "\ntsv2bam\n==========\n";
 
-    my $file_paths = "";
-    if (length($popmap_path) == 0) {
-        foreach $sample (@parents, @progeny, @samples) {
-            $file_paths .= " -s $sample->{'file'}";
-        }
-    }
+    if ($resume == true and tsv2bam_samples_exist($parents, $progeny, $samples, $out_path) == true) {
+        print STDERR  "  tsv2bam samples already exist, will use existing data.\n\n";
+        print $log_fh "tsv2bam samples already exist, will use existing data.\n\n";
 
-    $cmd = $exe_path . "tsv2bam -P $out_path $file_paths " . join(" ", @_tsv2bam);
-    print STDERR  "  $cmd\n\n";
-    print $log_fh "$cmd\n\n";
+    } else {
+        #
+        # If --resume is enabled and we ended up re-executing tsv2bam, force everything downstream to re-execute.
+        #
+        $force_reexe = true if ($resume == true);
+        
+        my $file_paths = "";
+        if (length($popmap_path) == 0) {
+            foreach $sample (@{$parents}, @{$progeny}, @{$samples}) {
+                $file_paths .= " -s $sample->{'file'}";
+            }
+        }
 
-    if (!$dry_run) {
-        open($pipe_fh, "$time $cmd 2>&1 |");
-        while (<$pipe_fh>) {
-            print $log_fh $_;
+        $cmd = $exe_path . "tsv2bam -P $out_path $file_paths " . join(" ", @_tsv2bam);
+        print STDERR  "  $cmd\n\n";
+        print $log_fh "$cmd\n\n";
+
+        if (!$dry_run) {
+            open($pipe_fh, "$time $cmd 2>&1 |");
+            while (<$pipe_fh>) {
+                print $log_fh $_;
+            }
+            close($pipe_fh);
+            check_return_value($?, $log_fh);
         }
-        close($pipe_fh);
-        check_return_value($?, $log_fh);
     }
 
     #
@@ -245,17 +327,23 @@ sub execute_stacks {
     print STDERR "Calling variants, genotypes and haplotypes...\n";
     print $log_fh "\ngstacks\n==========\n";
 
-    $cmd = $exe_path . "gstacks -P $out_path " . join(" ", @_gstacks);
-    print STDERR  "  $cmd\n\n";
-    print $log_fh "$cmd\n\n";
+    if ($resume == true and gstacks_output_exists($out_path) == true) {
+        print STDERR  "  gstacks catalog already exists, will use existing data.\n\n";
+        print $log_fh "gstacks catalog already exists, will use existing data.\n\n";
 
-    if (!$dry_run) {
-        open($pipe_fh, "$time $cmd 2>&1 |");
-        while (<$pipe_fh>) {
-            print $log_fh $_;
+    } else {
+        $cmd = $exe_path . "gstacks -P $out_path " . join(" ", @_gstacks);
+        print STDERR  "  $cmd\n\n";
+        print $log_fh "$cmd\n\n";
+
+        if (!$dry_run) {
+            open($pipe_fh, "$time $cmd 2>&1 |");
+            while (<$pipe_fh>) {
+                print $log_fh $_;
+            }
+            close($pipe_fh);
+            check_return_value($?, $log_fh);
         }
-        close($pipe_fh);
-        check_return_value($?, $log_fh);
     }
 
     printf(STDERR "Calculating population-level summary statistics\n");
@@ -278,8 +366,164 @@ sub execute_stacks {
     print $log_fh "denovo_map.pl is done.\n";
 }
 
+sub ustacks_sample_exists {
+    my ($sample, $out_path) = @_;
+
+    my $path = $out_path . "/" . $sample->{'file'} . ".alleles.tsv.gz";
+
+    # Check the file exists and is not empty.
+    return false if (!-e $path || -z _);
+
+    my $fh;
+    open($fh, "gunzip -c $path | tail -n 1 |") or die("Unable to read ustacks file, '$path'\n");
+    my $line = <$fh>;
+    close($fh);
+
+    chomp $line;
+
+    return true if ($line =~ /^# ustacks completed/);
+
+    return false;
+}
+
+sub catalog_exists {
+    my ($prev_completed_ustacks, $parents, $progeny, $samples, $out_path) = @_;
+
+    return false if ($force_reexe == true);
+
+    my $sample_cnt = 0;
+    
+    if ($genetic_map == true) {
+        $sample_cnt = scalar(@{$parents}) + scalar(@{$progeny});
+    } else {
+        $sample_cnt = scalar(@{$samples});
+    }
+
+    #
+    # If all ustacks samples have been processed, check if the catalog was completed.
+    #
+    if ($sample_cnt == scalar(@{$prev_completed_ustacks})) {
+        
+        my $path = $out_path . "/catalog.tags.tsv.gz";
+
+        # Check the file exists and is not empty.
+        return false if (!-e $path || -z _);
+
+        my $path = $out_path . "/catalog.snps.tsv.gz";
+        return false if (!-e $path || -z _);
+
+        my $path = $out_path . "/catalog.alleles.tsv.gz";
+        return false if (!-e $path || -z _);
+
+        my $fh;
+        open($fh, "gunzip -c $path | tail -n 1 |") or die("Unable to read catalog file, '$path'\n");
+        my $line = <$fh>;
+        close($fh);
+
+        chomp $line;
+
+        return true if ($line =~ /^# cstacks completed/);
+    }
+    
+    return false;
+}
+
+sub sstacks_samples_exist {
+    my ($parents, $progeny, $samples, $out_path) = @_;
+
+    return false if ($force_reexe == true);
+
+    my $sample_cnt    = scalar(@{$parents}) + scalar(@{$progeny}) + scalar(@{$samples});
+    my $completed_cnt = 0;
+
+    foreach $sample (@{$parents}, @{$progeny}, @{$samples}) {
+        my $path = $out_path . "/" . $sample->{'file'} . ".matches.tsv.gz";
+
+        # Check the file exists and is not empty.
+        return false if (!-e $path || -z _);
+
+        my $fh;
+        open($fh, "gunzip -c $path | tail -n 1 |") or die("Unable to read sstacks file, '$path'\n");
+        my $line = <$fh>;
+        close($fh);
+
+        chomp $line;
+
+        $completed_cnt++ if ($line =~ /^# sstacks completed/);
+    }
+
+    if ($sample_cnt == $completed_cnt) {
+        return true;
+    }
+    
+    return false;
+}
+
+sub tsv2bam_samples_exist {
+    my ($parents, $progeny, $samples, $out_path) = @_;
+
+    return false if ($force_reexe == true);
+    
+    my $sample_cnt    = scalar(@{$parents}) + scalar(@{$progeny}) + scalar(@{$samples});
+    my $completed_cnt = 0;
+
+    foreach $sample (@{$parents}, @{$progeny}, @{$samples}) {
+        my $path = $out_path . "/" . $sample->{'file'} . ".matches.bam";
+
+        # Check the file exists and is not empty.
+        $completed_cnt++ if (-e $path && !-z _);
+    }
+
+    if ($sample_cnt == $completed_cnt) {
+        my $path = $out_path . "/tsv2bam.log";
+
+        # Check the file exists and is not empty.
+        return false if (!-e $path || -z _);
+
+        my $fh;
+        open($fh, "tail -n 1 $path |") or die("Unable to read tsv2bam logfile, '$path'\n");
+        my $line = <$fh>;
+        close($fh);
+
+        chomp $line;
+
+        return true if ($line =~ /^tsv2bam is done./);
+    }
+
+    return false;
+}
+
+sub gstacks_output_exists {
+    my ($out_path) = @_;
+
+    return false if ($force_reexe == true);
+
+    my $path = $out_path . "/catalog.fa.gz";
+
+    # Check the file exists and is not empty.
+    return false if (!-e $path || -z _);
+
+    $path = $out_path . "/catalog.calls";
+    return false if (!-e $path || -z _);
+
+    $path = $out_path . "/gstacks.log";
+
+    return false if (!-e $path || -z _);
+
+    my $fh;
+    open($fh, "tail -n 1 $path |") or die("Unable to read tsv2bam logfile, '$path'\n");
+    my $line = <$fh>;
+    close($fh);
+
+    chomp $line;
+
+    return true if ($line =~ /^gstacks is done./);
+    
+    return false;
+}
+
 sub parse_population_map {
-    my ($sample_list, $pop_ids, $pops, $grp_ids, $grps) = @_;
+    my ($sample_list, $pop_ids, $pops, $grp_ids, $grps, $map) = @_;
 
     my ($fh, @parts, $line, $sample);
 
@@ -310,7 +554,7 @@ sub parse_population_map {
     }
 
     if (scalar(keys %{$grps}) == 0) {
-    $grps->{"1"}++;
+        $grps->{"1"}++;
 
         foreach $sample (@{$sample_list}) {
             $grp_ids->{$sample} = "1";
@@ -322,6 +566,18 @@ sub parse_population_map {
     print STDERR " and ", scalar(keys %{$grps});
     scalar(keys %{$grps}) == 1 ? print STDERR " group.\n" : print STDERR " groups.\n";
 
+    #
+    # Test for a genetic map.
+    #
+    my @k = keys %{$pops};
+    if (scalar(@k) == 2) {
+        if ( ($k[0] eq "parent"  && $k[1] eq "progeny") ||
+             ($k[0] eq "progeny" && $k[1] eq "parent") ) {
+            print STDERR "Genetic map detected from population map, only 'parent' samples will be loaded into the catalog.\n";
+            $$map = true;
+        }
+    }
+
     close($fh);
 }
 
@@ -346,6 +602,10 @@ sub initialize_samples {
                         $found = true;
                         $extension = "." . $suffixes[$i];
                         last;
+                    } elsif (!$paired && -e $sample_path . $sample . ".1." . $suffixes[$i]) {
+                        $found = true;
+                        $extension = ".1." . $suffixes[$i];
+                        last;
                     } elsif (-e $sample_path . $sample . ".1." . $suffixes[$i]) {
                         $found = true;
                         $extension = ".1." . $suffixes[$i];
@@ -371,27 +631,29 @@ sub initialize_samples {
             die("Error: Failed to open paired-end file '$path_pe'.\n") if ($paired && ! -e $path_pe);
             die("Unable to find an entry for '" . $sample . "' in the population map, '$popmap_path'.\n") if (!defined($pop_ids->{$sample}));
 
-            if ($pop_ids->{$sample} eq "parent") {
-                push(@{$parents}, {'path'   => $path,
-                                       'file'   => $sample,
-                                       'suffix' => $suffixes[$i],
-                                       'type'   => "parent",
-                                       'fmt'    => $fmts[$i],
+            if ($genetic_map == true) {
+                if ($pop_ids->{$sample} eq "parent") {
+                    push(@{$parents}, {'path'    => $path,
+                                       'file'    => $sample,
+                                       'suffix'  => $suffixes[$i],
+                                       'type'    => "parent",
+                                       'fmt'     => $fmts[$i],
                                        'path_pe' => $path_pe});
-            } elsif ($pop_ids->{$sample} eq "progeny") {
-                push(@{$progeny}, {'path'   => $path,
-                                       'file'   => $sample,
-                                       'suffix' => $suffixes[$i],
-                                       'type'   => "progeny",
-                                       'fmt'    => $fmts[$i],
+                } else {
+                    push(@{$progeny}, {'path'    => $path,
+                                       'file'    => $sample,
+                                       'suffix'  => $suffixes[$i],
+                                       'type'    => "progeny",
+                                       'fmt'     => $fmts[$i],
                                        'path_pe' => $path_pe});
+                }
             } else {
-                push(@{$samples}, {'path'   => $path,
-                                       'file'   => $sample,
-                                       'suffix' => $suffixes[$i],
-                                       'type'   => "sample",
-                                       'fmt'    => $fmts[$i],
-                                       'path_pe' => $path_pe});
+                push(@{$samples}, {'path'    => $path,
+                                   'file'    => $sample,
+                                   'suffix'  => $suffixes[$i],
+                                   'type'    => "sample",
+                                   'fmt'     => $fmts[$i],
+                                   'path_pe' => $path_pe});
             }
         }
     }
@@ -401,7 +663,7 @@ sub initialize_samples {
     #
     my %sample_hash;
 
-    foreach $sample (@{$samples}) {
+    foreach $sample (@{$samples}, @{$parents}, @{$progeny}) {
         $sample_hash{$sample->{'file'}}++;
 
         if (!defined($pop_ids->{$sample->{'file'}})) {
@@ -435,9 +697,13 @@ sub initialize_samples {
         }
     }
 
-    print STDERR "Found ", scalar(@{$parents}), " parental file(s).\n\n" if (scalar(@{$parents}) > 0);
-    print STDERR "Found ", scalar(@{$progeny}), " progeny file(s).\n\n" if (scalar(@{$progeny}) > 0);
-    print STDERR "Found ", scalar(@{$samples}), " sample file(s).\n\n" if (scalar(@{$samples}) > 0);
+    if (scalar(@{$samples}) > 0) {
+        print STDERR "Found ", scalar(@{$samples}), " sample file(s).\n\n";
+    } else {
+        print STDERR
+            "Found ", scalar(@{$parents}), " parental file(s).\n",
+            "Found ", scalar(@{$progeny}), " progeny file(s).\n\n";
+    }
 
     if ( scalar(@{$samples}) > 0 && (scalar(@{$parents}) > 0 || scalar(@{$progeny}) > 0) ) {
 	die("Both samples and parents/progeny were specified either on the command line (-s/-r/-p) or within the population map. Only one of the other may be specified.\n");
@@ -479,11 +745,11 @@ sub parse_command_line {
         $_ = shift @ARGV;
         if    ($_ =~ /^-v$/ || $_ =~ /^--version$/) { version(); exit 1; }
         elsif ($_ =~ /^-h$/) { usage(); }
-        elsif ($_ =~ /^-d$/ || $_ =~ /^--dry-run$/) { $dry_run   = true; }
-        elsif ($_ =~ /^-o$/)        { $out_path  = shift @ARGV; }
+        elsif ($_ =~ /^-d$/ || $_ =~ /^--dry-run$/)  { $dry_run   = true; }
+        elsif ($_ =~ /^-o$/ || $_ =~ /^--out-path$/) { $out_path  = shift @ARGV; }
         elsif ($_ =~ /^-e$/)        { $exe_path  = shift @ARGV; }
         elsif ($_ =~ /^-m$/)        { $min_cov     = shift @ARGV; }
-        elsif ($_ =~ /^-P$/)        { $min_rcov    = shift @ARGV; }
+        elsif ($_ =~ /^--resume$/)  { $resume      = true; }
         elsif ($_ =~ /^--paired$/)  { $paired      = true; }
         elsif ($_ =~ /^--samples$/) { $sample_path = shift @ARGV; }
         elsif ($_ =~ /^-O$/ || $_ =~ /^--popmap$/) {
@@ -494,10 +760,7 @@ sub parse_command_line {
             push(@_gstacks,     "-M " . $popmap_path);
             push(@_populations, "-M " . $popmap_path);
 
-        } elsif ($_ =~ /^-t$/) {
-            push(@_ustacks, "-d ");
-
-        } elsif ($_ =~ /^-T$/) {
+        } elsif ($_ =~ /^-T$/ || $_ =~ /^--threads/) {
             $arg = shift @ARGV;
             push(@_ustacks, "-p " . $arg);
             push(@_cstacks, "-p " . $arg);
@@ -570,11 +833,6 @@ sub parse_command_line {
     $exe_path = $exe_path . "/"          if (substr($exe_path, -1) ne "/");
     $out_path = substr($out_path, 0, -1) if (substr($out_path, -1) eq "/");
 
-    if (scalar(@parents) > 0 && scalar(@samples) > 0) {
-        print STDERR "You must specify either parent or sample files, but not both.\n";
-        usage();
-    }
-
     if (length($popmap_path) == 0) {
         print STDERR "You must specify a population map that lists your sample names (--popmap).\n";
         usage();
@@ -609,21 +867,22 @@ sub usage {
     version();
 
     print STDERR <<EOQ;
-denovo_map.pl --samples dir --popmap path -o dir [--paired [--rm-pcr-duplicates]] (assembly options) (filtering options) [-X prog:"opts" ...]
+denovo_map.pl --samples dir --popmap path --out-path dir [--paired [--rm-pcr-duplicates]] (assembly options) (filtering options) [-X prog:"opts" ...]
 
   Input/Output files:
     --samples: path to the directory containing the samples reads files.
     --popmap: path to a population map file (format is "<name> TAB <pop>", one sample per line).
-    o: path to an output directory.
+    -o,--out-path: path to an output directory.
 
   General options:
-    X: additional options for specific pipeline components, e.g. -X "populations: --min-maf 0.05".
-    T: the number of threads/CPUs to use (default: 1).
-    d: Dry run. Do not actually execute anything, just print the commands that would be executed.
+    -X: additional options for specific pipeline components, e.g. -X "populations: --min-maf 0.05".
+    -T, --threads: the number of threads/CPUs to use (default: 1).
+    --dry-run: dry run. Do not actually execute anything, just print the commands that would be executed.
+    --resume: resume executing the pipeline from a previous run.
 
   Stack assembly options:
-    M: number of mismatches allowed between stacks within individuals (for ustacks).
-    n: number of mismatches allowed between stacks between individuals (for cstacks; default 1; suggested: set to ustacks -M).
+    -M: number of mismatches allowed between stacks within individuals (for ustacks).
+    -n: number of mismatches allowed between stacks between individuals (for cstacks; default 1; suggested: set to ustacks -M).
 
   SNP model options:
     --var-alpha: significance level at which to call variant sites (for gstacks; default: 0.05).
