@@ -328,8 +328,8 @@ sub parse_command_line {
         $_ = shift @ARGV;
         if    ($_ =~ /^-v$/ || $_ =~ /^--version$/) { version(); exit 1; }
         elsif ($_ =~ /^-h$/) { usage(); }
-        elsif ($_ =~ /^-d$/ || $_ =~ /^--dry-run$/) { $dry_run = true; }
-        elsif ($_ =~ /^-o$/) { $out_path  = shift @ARGV; }
+        elsif ($_ =~ /^-d$/ || $_ =~ /^--dry-run$/)  { $dry_run = true; }
+        elsif ($_ =~ /^-o$/ || $_ =~ /^--out-path$/) { $out_path  = shift @ARGV; }
         elsif ($_ =~ /^-e$/) { $exe_path  = shift @ARGV; }
         elsif ($_ =~ /^--samples$/) {
             $sample_path = shift @ARGV;
@@ -359,6 +359,12 @@ sub parse_command_line {
             $arg = shift @ARGV;
             push(@_gstacks, "--gt-alpha " . $arg);
 
+        } elsif ($_ =~ /^-r$/ || $_ =~ /^--min-samples-per-pop$/) {
+            push(@_populations,   "--min-samples-per-pop " . shift @ARGV);
+
+        } elsif ($_ =~ /^-p$/ || $_ =~ /^--min-populations$/) {
+            push(@_populations,   "--min-populations " . shift @ARGV);
+
         } elsif ($_ =~ /^-X$/) {
             #
             # Pass an arbitrary command-line option to a pipeline program.
@@ -414,31 +420,35 @@ sub usage {
     version();
 
     print STDERR <<EOQ;
-ref_map.pl --samples path --popmap path [-s spacer] -o path [--rm-pcr-duplicates] [-X prog:"opts" ...]
+ref_map.pl --samples path --popmap path [-s spacer] --out-path path [--rm-pcr-duplicates] [-X prog:"opts" ...]
 
   Input/Output files:
     --samples: path to the directory containing the samples BAM (or SAM) alignment files.
     --popmap: path to a population map file (format is "<name> TAB <pop>", one sample per line).
-    s: spacer for file names: by default this is empty and the program looks for files
-       named 'SAMPLE_NAME.bam'; if this option is given the program looks for files
-       named 'SAMPLE_NAME.SPACER.bam'.
-    o: path to an output directory.
+    -s: spacer for file names: by default this is empty and the program looks for files
+        named 'SAMPLE_NAME.bam'; if this option is given the program looks for files
+        named 'SAMPLE_NAME.SPACER.bam'.
+    -o,--out-path: path to an output directory.
 
   General options:
-    X: additional options for specific pipeline components, e.g. -X "populations: -p 3 -r 0.50"
-    T: the number of threads/CPUs to use (default: 1).
-    d: Dry run. Do not actually execute anything, just print the individual pipeline commands
-       that would be executed.
+    -X: additional options for specific pipeline components, e.g. -X "populations: -p 3 -r 0.50"
+    -T: the number of threads/CPUs to use (default: 1).
+    -d: Dry run. Do not actually execute anything, just print the individual pipeline commands
+        that would be executed.
+
+  SNP model options:
+    --var-alpha: significance level at which to call variant sites (for gstacks; default: 0.05).
+    --gt-alpha: significance level at which to call genotypes (for gstacks; default: 0.05).
 
   Paired-end options:
     --rm-pcr-duplicates: remove all but one copy of read pairs of the same sample that have
                          the same insert length.
     --ignore-pe-reads: ignore paired-end reads even if present in the input
     --unpaired: ignore read pairing (for paired-end GBS; treat READ2's as if they were READ1's)
-                                  
-  SNP model options:
-    --var-alpha: significance level at which to call variant sites (for gstacks; default: 0.05).
-    --gt-alpha: significance level at which to call genotypes (for gstacks; default: 0.05).
+
+  Population filtering options:
+    -r,--min-samples-per-pop: minimum percentage of individuals in a population required to process a locus for that population (for populations; default: 0)
+    -p,--min-populations: minimum number of populations a locus must be present in to process a locus (for populations; default: 1)
 
   Miscellaneous:
     --time-components (for benchmarking)
