@@ -117,6 +117,7 @@ private:
 
 class BamHeader {
     bam_hdr_t* h_;
+    void init_sdict(); // (So that BamHeader::chrom_index can be const.)
 
     BamHeader& operator= (BamHeader&) = delete;
 
@@ -129,7 +130,7 @@ public:
     ~BamHeader()                            {destroy();}
     BamHeader& operator=(BamHeader&& other) {std::swap(h_, other.h_); return *this;}
     void reinit()                           {destroy(); h_ = bam_hdr_init();}
-    void reinit(htsFile* bam_f)             {destroy(); h_ = bam_hdr_read(bam_f->fp.bgzf);}
+    void reinit(htsFile* bam_f)             {destroy(); h_ = bam_hdr_read(bam_f->fp.bgzf); init_sdict();}
     void destroy()                          {if (h_!=NULL) {bam_hdr_destroy(h_); h_=NULL;}}
     bool empty() const                      {return h_==NULL;}
 
@@ -137,6 +138,7 @@ public:
     size_t      n_ref_chroms()           const {assert(h_); return h_->n_targets;}
     const char* chrom_str(int32_t index) const;
     size_t      chrom_len(int32_t index) const;
+    int32_t     chrom_index(const char* chrom_str) const;
 
     const bam_hdr_t* hts() const {return h_;}
           bam_hdr_t* hts()       {return h_;}
@@ -383,4 +385,15 @@ size_t BamHeader::chrom_len(int32_t index) const
     return h_->target_len[index];
 }
 
+inline
+int32_t BamHeader::chrom_index(const char* chrom_str) const
+{
+    assert(h_);
+    assert(h_->sdict);
+    int32_t index = bam_name2id(h_, chrom_str);
+    if (index < 0)
+        throw std::out_of_range("BamHeader::chrom_index");
+    return index;
+}
+
 #endif // __BAMI_H__
