@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2010-2018, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2010-2020, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -1127,6 +1127,21 @@ int write_catalog(map<int, CLocus *> &catalog) {
             write_simple_output(tag, tags, snps, alle);
     }
 
+    time(&rawtime);
+    timeinfo = localtime(&rawtime);
+    strftime(date, 32, "%F %T", timeinfo);
+    log.str("");
+    log << "# cstacks completed on " << date << "\n";
+    if (gzip) {
+        gzputs_throwing(gz_tags, log.str().c_str());
+        gzputs_throwing(gz_snps, log.str().c_str());
+        gzputs_throwing(gz_alle, log.str().c_str());
+    } else {
+        tags << log.str();
+        snps << log.str();
+        alle << log.str();
+    }
+
     if (gzip) {
         gzclose_throwing(gz_tags);
         gzclose_throwing(gz_snps);
