@@ -30,6 +30,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
+#include <cctype>
 #include <utility>
 #include <string>
 #include <iostream>
@@ -49,6 +50,7 @@
 #include "utils.h"
 #include "log_utils.h"
 #include "catalog_utils.h"
+#include "mapping_utils.h"
 #include "sql_utilities.h"
 #include "genotype_dictionaries.h"
 #include "ordered.h"
@@ -312,6 +314,7 @@ private:
     // Counters for external VCF
     size_t            _total_ext_vcf;
     vector<size_t>    _skipped_notsnp;
+    vector<size_t>    _skipped_notbinarysnp;
     vector<size_t>    _skipped_filter;
 
     // Controls for which loci are loaded
@@ -333,6 +336,7 @@ int     parse_command_line(int, char**);
 void    output_parameters(ostream &);
 void    open_log(ofstream &);
 int     build_file_list();
+bool    check_population_map_for_cross(const MetaPopInfo *);
 int     load_marker_list(string, set<int> &);
 int     load_marker_column_list(string, map<int, set<int> > &);
 //int     merge_shared_cutsite_loci(map<int, CSLocus *> &, PopMap<CSLocus> *, PopSum<CSLocus> *, map<int, pair<merget, int> > &, ofstream &);
