@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2010-2018, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2010-2020, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -1268,6 +1268,16 @@ write_matches(string sample_path, map<int, QLocus *> &sample)
         sstr.str("");
     }
 
+    time(&rawtime);
+    timeinfo = localtime(&rawtime);
+    strftime(date, 32, "%F %T", timeinfo);
+    log.str("");
+    log << "# sstacks completed on " << date << "\n";
+    if (in_file_type == FileT::gzsql)
+        gzputs_throwing(gz_matches, log.str().c_str());
+    else
+        matches << log.str();
+
     if (in_file_type == FileT::gzsql)
         gzclose_throwing(gz_matches);
     else
